/*

 RTF_alaska.prg

 23-11-2019

  COBA Systems Software
 BAST Open Source Project
 Business Account Software Technology
 www.cobasystems.com
 Program: Alaska Xbase++ 1.90.355 sl1
          XbTools, ADSDBE, eXpress++


-----------------------------------------------
UPIS RTF PODATKA U BAZU PODATAKA DBF ILI DRUGU
-----------------------------------------------
Data type for storing rich text format (RTF)
https://www.datanamic.com/support/storing-rtf-in-db.html
-----------------------------------------------
Vrsta podataka za spremanje bogatog tekstualnog formata (RTF)
Ovaj lanak e opisati koji tip podataka treba koristiti kada elite pohraniti
obogaeni tekst (RTF) u tablicu baze podataka.
Da biste donijeli ispravniju odluku o tome koji tip podataka koristiti za
spremanje datoteka obogaenog teksta u vaoj bazi podataka, vano je znati to
su datoteke obogaenog teksta. Podaci u datoteci obogaenog teksta zapravo jo
uvijek predstavljaju normalan tekst, ali sadre oznake za sve oblikovanje teksta.
Na primjer, Rich Text za niz koji je podebljan, u obliku kurzivnog slova, podvuen
u Times Roman 14 toaka, izgleda ovako:



{\rtf1\ansi\deff0\deftab720{\fonttbl{\f0\fswiss MS Sans Serif;}
{\f1\froman\fcharset2 Symbol;}{\f2\fswiss MS Sans Serif;}
{\f3\froman\fprq2 Times New Roman;}}
{\colortbl\red0\green0\blue0;}
\deflang1033\pard\plain\f3\fs28\b\i\ul Carpe diem.\plain\f2\fs17
\par }



Budui da je format obogaenog teksta ustvari tekst, moete ga pohraniti u stupac
s vrstom tekstualnih podataka. Imajte na umu duinu odabira. Kad je duina manja
od 4000 znakova, moete koristiti varchar. Kad je duljina vea od 4000 znakova,
bolje je potraiti alternativni tip podataka. Za baze podataka ORACLE postoji vrsta
podataka zvana CLOB koja se posebno uvodi za spremanje velikih znakovnih nizova.
Ostale baze podataka nude MEMO-ove, TEXT ....

*/





*---
* 23-11-2019
* METHOD RichText:IntlTranslate() ZAMENJEN SA TRANSLACIONOM TABELOM:
* FUNCTION RTF_coba_latin_cyrilic(nCHR) firme COBA Systems
*---
* 16-06-2008 COBA Systems  Stanojevi S.   izmene u kodu
* -----------------------------
* PROGRAM  : ALASRTF.PRG
* VERSION  : 1.00
* DATE     : 15 december 1999
* LANGUAGE : XBASE++
* AUTHOR   : PAUL C. Laney
* -----------------------------










/*----------------------------------------------------------------------------//
!short: Default parameters management */

#xcommand DEFAULT <uVar1> := <uVal1> ;
               [, <uVarN> := <uValN> ] => ;
                  <uVar1> := If( <uVar1> == nil, <uVal1>, <uVar1> ) ;;
                [ <uVarN> := If( <uVarN> == nil, <uValN>, <uVarN> ); ]



#include "RTF_richtext.ch"

#include "Xbp.ch"

CLASS RichText

   EXPORTED:
   VAR cFileName, hFile, nFontSize, nScale, lTrimSpaces, nFontNum
   VAR aTranslate, cTblHAlign, cCode

   VAR nTblFntNum, nTblFntSize, nTblRows, nTblColumns
   VAR nTblRHgt, aTableCWid, cRowBorder, cCellBorder, nCellPct
   VAR lTblNoSplit, nTblHdRows, nTblHdHgt, nTblHdPct, nTblHdFont
   VAR nTblHdFSize
   VAR cCellAppear, cHeadAppear
   VAR cCellHAlign, cHeadHAlign
   VAR nCurrRow, nCurrColumn


   METHOD Appearance
   METHOD BorderCode
   METHOD Borders

   INLINE METHOD CloseGroup()
      Fwrite( ::hFile, "}" )
   RETURN (Self)

   METHOD DefineTable
   METHOD FormatCode
   METHOD HAlignment
   METHOD Init          // Initialize the RichText Object
   METHOD IntlTranslate
   METHOD LineSpacing
   METHOD LogicCode
   METHOD NewBase
   METHOD Newfont       // Change the current font
   METHOD NewSection
   METHOD Numcode       // Write an RTF code with a numeric parameter
                        // to the output file

   INLINE METHOD OpenGroup()
      Fwrite( ::hFile, "{" )
   RETURN (Self)

   METHOD PageNumber
   METHOD PageSetup
   METHOD Paragraph
   METHOD SetFontSize
   METHOD ValToChar
   METHOD Write
   METHOD WriteCell


   INLINE METHOD TextCode( cCode )
      Fwrite(::hFile, ::FormatCode(cCode) )
   RETURN Self

   INLINE METHOD xEnd()
      ::TextCode( "par\pard" )
      ::CloseGroup()
      Fclose(::hFile)
   RETURN Self

   INLINE METHOD BeginHeader()
      ::OpenGroup()
      ::TextCode("header \pard")
   Return Self

   INLINE METHOD EndHeader()
      ::TextCode("par")
      ::CloseGroup()
   Return Self

   INLINE METHOD BeginFooter()
      ::OpenGroup()
      ::TextCode("footer \pard")
   Return Self

   INLINE METHOD EndFooter()
      ::TextCode("par")
      ::CloseGroup()
   Return Self

   INLINE METHOD BeginRow()
     ::TextCode( "trowd" )
     ::nCurrRow += 1
   Return Self

   INLINE METHOD EndRow()
     ::TextCode( "row" )
   Return Self

ENDCLASS

* ----------------------------------------------------------------------
  METHOD RichText:Init( cFileName, aFontData, nFontSize, nScale, ;
                        aHigh, lWarn, lGetFile, cPath )
* ----------------------------------------------------------------------
*
* Description:  Initialize the RichText Object
* Arguments:
* Return:
*
* ----------------------------------------------------------------------

LOCAL i, lOK := .T.
LOCAL cTopFile  := "rtf1\ansi\ansicpg1250\deff0\deftab720"      // COBA
LOCAL cColors   := "colortbl\red0\green0\blue0;"
LOCAL cAuthor   := "info\author RichText() Class for Xbase++"
LOCAL cGetTitle := "Enter a File Name for the Report"




DEFAULT ;
   cFileName := "REPORT.RTF", ; //   aFontData := { "Courier New" }, ;
   aFontData := { "Courier" }, ;
   nFontSize := 12, ;
   nScale    := INCH_TO_TWIP, ;
   lWarn     := .F., ;
   lGetFile  := .F.

   ::cFileName := cFileName
   ::nFontSize := nFontSize
   ::nScale    := nScale
   ::hFile     := -2

   ::lTrimSpaces := .F.

   IF VALTYPE(aHigh) == "A"
      ::aTranslate := aHigh
   ELSE
      ::IntlTranslate()
   ENDIF

   IF !EMPTY(cPath)
      ::cFileName := cPath + ::cFileName
   ENDIF

   IF lGetFile
      ::cFileName := PC_GetSaveFile( ::cFileName )
   ENDIF

   IF !EMPTY( ::cFileName )

      // If no extension specified in file name, use ".RTF"
      IF !("." $ ::cFileName)
         ::cFileName += ".rtf"
      ENDIF

      IF lWarn .AND. File( ::cFileName )
         lOk := TdYesNo( "Fajl " + PC_DotFile( ::cFileName ) + ;
                " postoji i zamenjuje se ?" )
      ENDIF

      IF lOk
         // Create/open a file for writing
         ::hFile := FCreate( ::cFileName )
      ENDIF

   ELSE
      lOK := .F.
   ENDIF

   IF ::hFile >= 0

      // Generate RTF file header
      // This opens the top-most level group for the report
      // This group must be explicitly closed by the application!

      // Generiranje zaglavlja RTF datoteke
      // Ovo otvara grupu na najviem nivou za izvetaj
      // Ova grupa mora biti izriito zatvorena aplikacijom!

      ::OpenGroup()

      ::TextCode( cTopFile )

      // Generate a font table, and write it to the header
      ::nFontNum := LEN(aFontData)
      ::OpenGroup()
      ::TextCode( "fonttbl" )
      FOR i := 1 TO ::nFontNum
         ::OpenGroup()
         ::NewFont( i )
         ::TextCode( "fnil" )
         ::Write( aFontData[i] + ";" )
         ::CloseGroup()
      NEXT
      ::CloseGroup()

      // Use default color info, for now...
      ::OpenGroup()
      ::TextCode( cColors )
      ::CloseGroup()

      // Add file author info
      ::OpenGroup()
      ::TextCode( cAuthor )
      ::CloseGroup()

      // NOTE:  At this point, we have an open group (the report itself)
      // that must be closed at the end of the report.

   ENDIF

RETURN (Self)

* ----------------------------------------------------------------------
  METHOD RichText:NewFont( nFontNumber )
* ----------------------------------------------------------------------
*
* Description:  Change the current font.
*               Converts app-level font number into RTF font number.
* Arguments:
* Return:
*
* ----------------------------------------------------------------------

   IF !Empty( nFontNumber ) .AND. nFontNumber <= ::nFontNum
      ::NumCode( "f", nFontNumber-1, .F. )
   ENDIF

RETURN (Self)

* ----------------------------------------------------------------------
  METHOD RichText:NumCode( cCode, nValue, lScale )
* ----------------------------------------------------------------------
*
* Description:  Write an RTF code with a numeric parameter
*               to the output file
*
*               NOTE: Most RTF numeric measurements must be specified
*               in "Twips" (1/20th of a point, 1/1440 of an inch).
*               However, the interface layer of the RichText class
*               defaults to accept inches.  Therefore, all such
*               measurements must be converted to Twips.
*
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
LOCAL cWrite := ""

   IF Valtype( cCode ) == "C" .AND. Valtype( nValue ) == "N"

      cCode := ::FormatCode( cCode )

      cWrite += cCode

      DEFAULT lScale := .T.
      IF lScale
         nValue := Int( nValue * ::nScale )
      ENDIF
      cWrite += Alltrim( str( nValue ) )

      FWrite(::hFile, cWrite )

   ENDIF

RETURN (Self)

* ----------------------------------------------------------------------
  METHOD RichText:Write( xData, lCodesOK )
* ----------------------------------------------------------------------
*********************************************************************
* Description:  Write data to output file, accounting for any characters
*               above ASCII 127 (RTF only deals with 7-bit characters
*               directly) -- 8-bit characters must be handled as hex data.
* Arguments:
* Return:
*
*--------------------------------------------------------------------
* Date       Developer   Comments
* 01/06/97   TRM         Creation
*********************************************************************
LOCAL cWrite := ""
LOCAL i, cChar, nChar
LOCAL cString := ::ValToChar( xData )
LOCAL aCodes := { "\", "{", "}" }
LOCAL aReturn := { CHR(13), CHR(10) }

DEFAULT lCodesOK := .F.



IF ::lTrimSpaces
	cString := RTRIM( cString )
ENDIF

//cString := " " + cString

FOR i := 1 TO LEN(cString)

	cChar := SUBSTR(cString, i, 1)
	nChar := ASC(cChar)

	IF nChar < 128

		IF nChar > 91

			// Process special RTF symbols
			IF !lCodesOK
				IF ASCAN( aCodes, cChar ) > 0
					cChar := "\" + cChar
				ENDIF
			ENDIF

		ELSEIF nChar < 33

			// 7/9/97
			// 1. Don't convert hard returns to "\par" if we're processing
			//    RTF data (i.e., lCodesOK), since some RTF's have
			//    stray returns in them.
			// 2. Convert all tabs to "\tab", since MS-Word doesn't like CHR(9)'s

			IF !lCodesOK .AND. nChar == 13 // Turn carriage returns into new paragraphs
				cChar := "\par "
			ELSEIF nChar == 9 // 7/9/97 -- Convert tabs
				cChar := "\tab "
			ELSEIF nChar == 10 // Ignore line feeds
				LOOP
			ENDIF

		ENDIF

		cWrite += cChar


	ELSE  // extended karakter set Ǝ  i irilica mora da ima tabelu za translaciju

		// We have a high-order character, which is a no-no in RTF.
		// If no international translation table for high-order characters
		// is specified, write data verbatim in hex format.  If a
		// translation table is specified, look up the appropriate
		// hex value to write.

                // Imamo karakter visokog reda, koji je ne-ne u RTF-u.
                // Ako nema meunarodne tablice za prijevod znakova visokog reda
                // je specificirano, zapisati podatke doslovno u hex formatu. Ako a
                // Tabela prevoenja je navedena, potraite odgovarajuu
                // hex vrijednost za pisanje.



		IF EMPTY( ::aTranslate )
			// Ignore soft line breaks
			IF nChar == 141
				LOOP
			ELSE
				cWrite += "'" + LOWER( ::NewBase( nChar, 16 ) )
			ENDIF
		ELSE
			* stop(cChar,asc(cChar),asc(cChar)-127,1) // Ǝ  // COBA

			******************************************  // 15 april 1998  Paul C. Laney
			* cWrite += ::aTranslate[ ASC(cChar)-127 ]  // vraa hex string u formatu:  "\'8e"
                        ******************************************  // koristi doraenu P.C.Laney tabelu za konverziju
                                                                    // METHOD RichText:IntlTranslate()

                        *****************************************   // 23-11-2019 COBA Systems
                          cWrite += ::aTranslate[ ASC(cChar) ]      // vraa hex string u formatu:  "\'8e"
                        *****************************************   // koristi latin-cyrilic tabelu za konverziju
                                                                    // METHOD RichText:IntlTranslate()

			* msgbox(cWrite)  // * vraa za "Ǝ" := "\'8a" + "\'d0" + "\'c8" + "\'c6" + "\'8e"
		ENDIF

	ENDIF

NEXT

//cWrite := cWrite+"  " // COBA

::OpenGroup()
FWRITE(::hFile, cWrite )
::CloseGroup()

*************************  END OF Write()  **************************
RETURN self


METHOD RichText:ValToChar( x )

   IF valtype( x ) == "C"

      RETURN ( x )

   ELSEIF valtype( x ) == "M"

      RETURN ( memotran( x, chr(13)+chr(10), " " ) )

   ELSEIF valtype( x ) == "N"

      RETURN alltrim( str( x ) )

   ELSEIF valtype( x ) == "D"

      RETURN dtoc( x )

   ELSEIF valtype( x ) == "L"

      Return if( x, ".T.", ".F." )

   ENDIF

RETURN ( "Error in function cValtype" )

METHOD RichText:NewBase( nDec, nBase )
*********************************************************************
* Description:  Convert a decimal numeric to a string in another
*               base system
* Arguments:
* Return:
*
*--------------------------------------------------------------------
* Date       Developer   Comments
* 01/12/97   TRM         Creation
*
*********************************************************************
LOCAL cNewBase := "", nDividend, nRemain, lContinue := .T., cRemain

DO WHILE lContinue

	nDividend := INT( nDec / nBase )
	nRemain := nDec % nBase

	IF nDividend >= 1
		nDec := nDividend
	ELSE
		lContinue := .F.
	ENDIF

	IF nRemain < 10
		cRemain := ALLTRIM(STR(nRemain,2,0))
	ELSE
		cRemain := CHR( nRemain + 55 )
	ENDIF

	cNewBase := cRemain + cNewBase

ENDDO

RETURN cNewBase
************************  END OF NewBase()  *************************


METHOD RichText:FormatCode( cCode )
*********************************************************************
* Description:  Remove extraneous spaces from a code, and make sure
*               that it has a leading backslash ("\").
* Arguments:
* Return:
*
*--------------------------------------------------------------------
* Date       Developer   Comments
* 01/12/97   TRM         Creation
*
*********************************************************************
cCode := ALLTRIM(cCode)
IF !( LEFT(cCode, 1) == "\" )
	cCode := "\" + cCode
ENDIF

RETURN cCode
***********************  END OF FormatCode()  ***********************

* ----------------------------------------------------------------------
  METHOD RichText:PageSetup( nLeft, nRight, nTop, nBottom, nWidth, nHeight, ;
				nTabWidth, lLandscape, lNoWidow, cVertAlign, ;
				cPgNumPos, lPgNumTop )
* ----------------------------------------------------------------------
*********************************************************************
* Description:  Define default page setup info for file
*               This information is placed in the "document formatting
*               group" of the RTF file, except for vertical alignment,
*               which, if supplied, is treated as a new section.
* Arguments:
* Return:
* ----------------------------------------------------------------------
DEFAULT lLandscape := .F.
DEFAULT lNoWidow := .F.
DEFAULT lPgNumTop := .F.


// Note -- "landscape" should not be specified here if landscape and
// portrait orientations are to be mixed.  If "landscape' is specified,
// the paper width and height should also be specified, and consistent
// (i.e., with landscape/letter, width==11 and height==8.5)

::LogicCode( "landscape", lLandscape )
::NumCode( "paperw", nWidth )
::NumCode( "paperh", nHeight )

::LogicCode( "widowctrl", lNoWidow )
::NumCode( "margl", nLeft )
::NumCode( "margr", nRight )
::NumCode( "margt", nTop )
::NumCode( "margb", nBottom )
::NumCode( "deftab", nTabWidth )


// Vertical alignment and page number position are "section-specific"
// codes.  But we'll put them here anyway for now...

IF !EMPTY( cVertAlign )
	::TextCode( "vertal" + LOWER( LEFT(cVertAlign,1) ) )
ENDIF

// Set the initial font size
::SetFontSize(::nFontSize)

// Forget page numbers for now...


RETURN Self

**********************  END OF PageSetup()  *************************

* ----------------------------------------------------------------------
  METHOD RichText:LogicCode( cCode, lTest )
* ----------------------------------------------------------------------
*
* Description:  Write an RTF code if the supplied value is true
*
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
IF VALTYPE(cCode) == "C" .AND. VALTYPE(lTest) == "L"
	IF lTest
		::TextCode( cCode )
	ENDIF
ENDIF

RETURN Self

***********************  END OF LogicCode()  *************************

* ----------------------------------------------------------------------
  METHOD RichText:SetFontSize( nFontSize )
* ----------------------------------------------------------------------
*
* Description:    Size in points -- must double value because
*                 RTF font sizes are expressed in half-points
* Arguments:
* Return:
*
* ----------------------------------------------------------------------

IF VALTYPE( nFontSize ) == "N"
	::nFontSize := nFontSize
	::NumCode( "fs", ::nFontSize*2, .F. )
ENDIF

RETURN Self
**********************  END OF SetFontSize()  ***********************

* ----------------------------------------------------------------------
  METHOD RichText:Paragraph( cText, nFontNumber, nFontSize, cAppear, ;
		cHorzAlign, aTabPos, nIndent, nFIndent, nRIndent, nSpace, ;
		lSpExact, nBefore, nAfter, lNoWidow, lBreak, ;
		lBullet, cBulletChar, lHang, lDefault, lNoPar, ;
		nTCLevel )
* ----------------------------------------------------------------------
*
* Description:  Write a new, formatted paragraph to the output file.
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
DEFAULT ;
	lDefault := .F., ;
	lNoWidow := .F., ;
	lBreak := .F., ;
	lBullet := .F., ;
	lHang := .F., ;
	cAppear := "", ;
	cHorzAlign := "", ;
	cBulletChar := "\bullet", ;
	lNoPar := .F.

::LogicCode("pagebb", lBreak)

IF !lNoPar
	::TextCode( "par" )
ENDIF

::LogicCode( "pard", lDefault )
::NewFont( nFontNumber )
::SetFontSize( nFontSize )
::Appearance( cAppear )
::HAlignment( cHorzAlign )

IF VALTYPE( aTabPos ) == "A"
	AEVAL( aTabPos, { |x| ::NumCode("tx", x) } )
ENDIF

::NumCode( "li", nIndent )
::NumCode( "fi", nFIndent )
::NumCode( "ri", nRIndent )
::LineSpacing( nSpace, lSpExact )

::NumCode( "sb", nBefore )
::NumCode( "sa", nAfter )

::LogicCode("keep", lNoWidow)

IF lBullet
	::OpenGroup()
		::TextCode( "*" )
		::TextCode( "pnlvlblt" )
		::LogicCode( "pnhang", lHang )
		::TextCode( "pntxtb " + cBulletChar )
	::CloseGroup()
ENDIF

::Write( cText )

// 7/9/97
IF VALTYPE( nTCLevel ) == "N"
	::OpenGroup()
		::TextCode( "v" ) // this hides the following text
		::TextCode( "tc" )
		::Write( cText )
		::NumCode( "tcl", nTCLevel, .F. )
	//	::TextCode( "v0" ) // this turns off hidden attribute
	::CloseGroup()
ENDIF

RETURN Self
**********************  END OF Paragraph()  *************************

* ----------------------------------------------------------------------
  METHOD RichText:Appearance( cAppear )
* ----------------------------------------------------------------------
* Description:  Change the "appearance" (bold, italic, etc.)
*               Appearance codes are concatenable at the app-level
*               and already contain backslashes.
* Arguments:
* Return:
*
* ----------------------------------------------------------------------

// Special case (see .CH file) -- first remove leading slash ...ugh.
IF !EMPTY(cAppear)
	::TextCode( SUBSTR(cAppear, 2) )
ENDIF

RETURN Self
***********************  END OF Appearance()  ***********************

* ----------------------------------------------------------------------
  METHOD RichText:HAlignment( cAlign )
* ----------------------------------------------------------------------
*
* Description:  Change the horizontal alignment
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
IF !EMPTY(cAlign)
	::TextCode( "q" + LOWER(LEFT(cAlign,1)) )
ENDIF

RETURN Self
**********************  END OF HAlignment()  ************************

* ----------------------------------------------------------------------
  METHOD RichText:LineSpacing( nSpace, lSpExact )
* ----------------------------------------------------------------------
*
* Description:  Change the line spacing (spacing can either be "exact"
*               or "multiple" (of single spacing).  If exact, the units
*               of the supplied value must be converted to twips.
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
DEFAULT lSpExact := .F.

::NumCode( "sl", nSpace, lSpExact )
IF !EMPTY( nSpace )
	::NumCode( "slmult", IIF( lSpExact, 0, 1 ), .F. )
ENDIF

RETURN Self
**********************  END OF LineSpacing()  ***********************

* ----------------------------------------------------------------------
  METHOD RichText:PageNumber( cHorzAlign )
* ----------------------------------------------------------------------
*
* Description:  Insert a page number field.
*               Best used within headers and footers.
* Arguments:
* Return:
*
* ----------------------------------------------------------------------

DEFAULT cHorzAlign := ""

::OpenGroup()

	::HAlignment( cHorzAlign ) // not sure if this works

	::TextCode( "field" )
	::OpenGroup()
		::TextCode( "*\fldinst PAGE  \" )
		::TextCode( "* MERGEFORMAT  " )
	::CloseGroup()
	::OpenGroup()
		::TextCode( "fldrslt " ) // don't include a result for now
	::CloseGroup()
::CloseGroup()


RETURN Self
************************  END  OF PageNumber()  *********************

* ----------------------------------------------------------------------
  METHOD RichText:NewSection( lLandscape, nColumns, nLeft, nRight, nTop, nBottom, ;
				nWidth, nHeight, cVertAlign, lDefault )
* ----------------------------------------------------------------------
*
* Description:  Open a new section, with optional new formatting
*               properties.
*
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
DEFAULT lDefault := .F.

//::OpenGroup()
::TextCode( "sect" )

IF lDefault
	::TextCode( "sectd" )
ENDIF

::LogicCode( "lndscpsxn", lLandscape )
::NumCode( "cols", nColumns, .F. )
::NumCode( "marglsxn", nLeft )
::NumCode( "margrsxn", nRight )
::NumCode( "margtsxn", nTop )
::NumCode( "margbsxn", nBottom )
::NumCode( "pgwsxn", nWidth )
::NumCode( "pghsxn", nHeight )

IF !EMPTY( cVertAlign )
	::TextCode( "vertal" + LOWER( LEFT(cVertAlign,1) ) )
ENDIF

RETURN Self
***********************  END OF NewSection()  **********************

* ----------------------------------------------------------------------
  METHOD RichText:DefineTable( cTblHAlign, nTblFntNum, nTblFntSize, ;
		cCellAppear, cCellHAlign, nTblRows, ;
		nTblColumns, nTblRHgt, aTableCWid, cRowBorder, cCellBorder, nCellPct, ;
		lTblNoSplit, nTblHdRows, nTblHdHgt, nTblHdPct, nTblHdFont, ;
		nTblHdFSize, cHeadAppear, cHeadHAlign )
* ----------------------------------------------------------------------
*
* Description:  Define the default setup for a table.
*               This simply saves the parameters to the object's
*               internal instance variables.
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
LOCAL i
DEFAULT ;
	cTblHAlign := "CENTER", ;
	nTblFntNum := 1, ;
	nTblFntSize := ::nFontSize, ;
	nTblRows := 1, ;
	nTblColumns:= 1, ;
	nTblRHgt := NIL, ;
	aTableCWid := ARRAY(nTblColumns), ; // see below
	cRowBorder := "NONE", ;
	cCellBorder := "SINGLE", ;
	nCellPct := 0, ;
	lTblNoSplit := .F., ;
	nTblHdRows := 0, ;
	nTblHdHgt := nTblRHgt, ;
	nTblHdPct := .1, ;
	nTblHdFont := nTblFntNum, ;
	nTblHdFSize := ::nFontSize + 2

	IF aTableCWid[1] == NIL
		AFILL( aTableCWid, 6.5/nTblColumns )
	ELSEIF VALTYPE(aTableCWid[1]) == "A"
		aTableCWid := ACLONE(aTableCWid[1])
	ENDIF

	// Turn independent column widths into "right boundary" info...
	FOR i := 2 TO LEN( aTableCWid )
		aTableCWid[i] += aTableCWid[i-1]
	NEXT

::cTblHAlign := LOWER( LEFT(cTblHAlign, 1) )
::nTblFntNum := nTblFntNum
::nTblFntSize := nTblFntSize
::cCellAppear := cCellAppear
::cCellHAlign := cCellHAlign
::nTblRows := nTblRows
::nTblColumns:= nTblColumns
::nTblRHgt := nTblRHgt
::aTableCWid := aTableCWid
::cRowBorder := ::BorderCode( cRowBorder )
::cCellBorder := ::BorderCode( cCellBorder )
::nCellPct := IIF( nCellPct < 1, nCellPct*10000, nCellPct*100 )
::lTblNoSplit := lTblNoSplit
::nTblHdRows := nTblHdRows
::nTblHdHgt := nTblHdHgt
::nTblHdPct := IIF( nTblHdPct < 1, nTblHdPct*10000, nTblHdPct*100 )
::nTblHdFont := nTblHdFont
::nTblHdFSize := nTblHdFSize
::cHeadAppear := cHeadAppear
::cHeadHAlign := cHeadHAlign

::nCurrColumn := 0
::nCurrRow    := 0

RETURN Self
**********************  END OF DefineTable()  ***********************

* ----------------------------------------------------------------------
  METHOD RichText:BorderCode( cBorderID )
* ----------------------------------------------------------------------
*
* Description:  Convert an application-level border ID into
*               a valid RTF border code.
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
LOCAL cBorderCode := "", n
LOCAL aBorder := ;
	{ ;
	{ "NONE",        NIL   }, ;
	{ "SINGLE",      "s"   }, ;
	{ "DOUBLETHICK", "th"  }, ;
	{ "SHADOW",      "sh"  }, ;
	{ "DOUBLE",      "db"  }, ;
	{ "DOTTED",      "dot" }, ;
	{ "DASHED",      "dash"}, ;
	{ "HAIRLINE",    "hair"}  ;
	}

cBorderID := UPPER( RTRIM(cBorderID) )

n := ASCAN( aBorder, { |x| x[1] == cBorderID } )

IF n > 0
	cBorderCode := aBorder[n][2]
ENDIF

RETURN cBorderCode
************************  END  OF BorderCode()  *********************

* ----------------------------------------------------------------------
  METHOD RichText:WriteCell( cText, nFontNumber, nFontSize, cAppear, cHorzAlign, ;
				nSpace, lSpExact, cCellBorder, nCellPct, lDefault, ;
				lMrgColumns, nMrgColumns )
* ----------------------------------------------------------------------
*
* Description:  Write a formatted cell of data to the current row
*               of the current table.  Also takes care of the logic
*               required for headers & header formatting.
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
LOCAL i

DEFAULT ;
	cText := "", ;
	lDefault := .F., ;
	lMrgColumns := .F., ;
	nMrgColumns := ::nTblColumns

// Increment/reset the column #
IF ::nCurrColumn == ::nTblColumns
	::nCurrColumn := 1
ELSE
	::nCurrColumn += 1
ENDIF


// Apply any one-time formatting for header/body

IF ::nCurrColumn == 1

	IF ::nCurrRow == 0 .AND. ::nTblHdRows > 0

		// Start a separate group for the header rows
		::OpenGroup()
		::BeginRow()

		// We need to apply header formats
		// The "\trgaph108" & "trleft-108" are the defaults used by MS-Word,
		// so if it's good enough for Word, it's good enough for me...

		::TextCode( "trgaph108\trleft-108" )
		::TextCode( "trq" + ::cTblHAlign )
		::Borders( "tr", ::cRowBorder )
		::NumCode( "trrh", ::nTblHdHgt )
		::TextCode( "trhdr" )
		::LogicCode( "trkeep", ::lTblNoSplit )

		// Set the default border & width info for each header cell
		FOR i := 1 TO LEN( ::aTableCWid )
			::NumCode( "clshdng", ::nTblHdPct, .F. )
			::Borders( "cl", ::cCellBorder )
			::NumCode("cellx", ::aTableCWid[i] )
		NEXT

		// Identify the header-specific font
		::NewFont( ::nTblHdFont )
		::SetFontSize( ::nTblHdFSize )
		::Appearance( ::cHeadAppear )
		::HAlignment( ::cHeadHAlign )

		::TextCode( "intbl" )

	ELSEIF ::nCurrRow == ::nTblHdRows

		// The header rows are over,
		// so we need to apply formats to the body of the table.

		// First close the header section, if one exists
		IF ::nTblHdRows > 0
			::EndRow()
			::CloseGroup()
		ENDIF

		::BeginRow()
		::TextCode( "trgaph108\trleft-108" )
		::TextCode( "trq" + ::cTblHAlign )
		::Borders( "tr", ::cRowBorder )
		::NumCode( "trrh", ::nTblRHgt )
		::LogicCode( "trkeep", ::lTblNoSplit )

		// Set the default shading, border & width info for each body cell
		FOR i := 1 TO LEN( ::aTableCWid )
			::NumCode( "clshdng", ::nCellPct, .F. )
			::Borders( "cl", ::cCellBorder )
			::NumCode("cellx", ::aTableCWid[i] )
		NEXT

		// Write the body formatting codes
		::NewFont( ::nTblFntNum )
		::SetFontSize( ::nTblFntSize )
		::Appearance( ::cCellAppear )
		::HAlignment( ::cCellHAlign )

		::TextCode( "intbl" )

	ELSE

		// End of a row of the table body.
		::EndRow()

		// Prepare the next row for inclusion in table
		::TextCode( "intbl" )

	ENDIF

ENDIF



// Apply any cell-specific formatting, and write the text

::OpenGroup()

	::LogicCode( "pard", lDefault )
	::NewFont( nFontNumber )
	::SetFontSize( nFontSize )
	::Appearance( cAppear )
	::HAlignment( cHorzAlign )
	::LineSpacing( nSpace, lSpExact )
	::Borders( "cl", cCellBorder )
	::NumCode( "clshdng", nCellPct, .F. )

	// Now write the text
	::Write( cText )

::CloseGroup()

// Close the cell
::TextCode( "cell" )

RETURN Self
***********************  END OF WriteCell()  ************************

* ----------------------------------------------------------------------
  METHOD RichText:Borders( cEntity, cBorder )
* ----------------------------------------------------------------------
*
* Description:  Apply borders to rows or cells.  Currently limited to
*               one type of border per rectangle.
* Arguments:
* Return:
*
* ----------------------------------------------------------------------
LOCAL i, aBorder := { "t", "b", "l", "r" }

IF VALTYPE( cBorder ) == "C"
	FOR i := 1 TO 4
		::TextCode( cEntity + "brdr" + aBorder[i] + "\brdr" + cBorder )
	NEXT
ENDIF

RETURN Self
************************  END OF Borders()  *************************

FUNCTION RTFMerge( cPriFile, nSkip, xRTF, cClipDelims )
*********************************************************************
* Description:  Merge data into the format specified in <cPriFile>,
*               which contains Clipper expressions delimited by
*               <cClipDelims>.  The intent is that <cPriFile> will
*               be an RTF file, though it doesn't have to be.
*
*               <nSkip> specifies the number of bytes to skip at
*               the top of the file, before reading the merge data.
*               This allows you to skip the RTF header, for example,
*               so that the merged output doesn't have multiple headers.
*
*               <xRTF> may be either a RichText() object, or just an
*               open file handle.
*
*               <cClipDelims> defaults to "<[]>".  If specified, it
*               must be exactly 4 characters (4 make it easier to
*               specify unique delimiters).  The first two bytes
*               specify the opening delimiter, and the last two specify
*               the closing delimiter. These are the delimiters that
*               must be used in <cPriFile> to designate Clipper
*               expressions.  For example <[Myarea->Name]>, or
*               <[DBSKIP()]>.  The supplied expressions must be
*               macro-compilable.  Currently, no error handling is
*               supplied for bad expressions.
*
*               Inspired by a request from Michael Mozina on the
*               Compuserve Clipper Forum.
*
* Opis: Spajanje podataka u format naveden u <cPriFile>,
* koji sadri Clipper izraze ograniene s
* <cClipDelims>. Namjera je da <cPriFile> hoe
* biti RTF datoteka, iako to ne mora biti.
*
* <nSkip> odreuje broj bajtova na kojima treba preskoiti
* vrh datoteke, prije itanja podataka o spajanju.
* To vam omoguava da preskoite RTF zaglavlje, na primjer,
* tako da spojeni izlaz nema vie zaglavlja.
*
* <xRTF> moe biti ili objekt RichText () ili samo an
* ruka otvorene datoteke.
*
* <cClipDelims> podrazumevano je postavljen na "<[]>". Ako je specificirano, to
* mora imati tano 4 znaka (4 olakavaju to)
* navedite jedinstvene razgraniitelje). Prva dva bajta
* odredite poetni razgraninik, a posljednja dva odredite
* razdjelnik zatvaranja. To su razdvajai koji
* mora se koristiti u <cPriFile> za oznaavanje Clipper-a
* izrazi. Na primjer, <[Myarea-> Name]> ili
* <[DBSKIP ()]>. Priloeni izrazi moraju biti
* makrokompolirati. Trenutno nema rukovanja grekama
* isporuuje se za loe izraze.
*
* Inspiriran zahtevom Majkla Mozine o
* Compuserve Clipper Forum.


* Arguments:
* Return:
*
*--------------------------------------------------------------------
* Date       Developer   Comments
* 02/02/97   TRM         Creation
*
*********************************************************************
LOCAL hPriFile, aDelims, cBuffer := "", cPrevBuff := "", cOrigBuff := ""
LOCAL nCurrCode := 0, cExpr := "", lCodeTest
LOCAL lRTF := VALTYPE(xRTF) == "O"
LOCAL xEval
LOCAL nRead := 0, nTotBytes := 0

DEFAULT cClipDelims := "<[]>"

hPriFile := FOPEN( cPriFile )

IF hPriFile < 0
	TdMsg( "Could not open primary merge file: " + PC_DotFile(cPriFile) + "." )
ELSE

	IF !( LEN(cCLipDelims) == 4 )

		TdMsg( "Bad delimiters specified for merge file." )

	ELSE

		// File is open, and delimiters are OK, so let's move forward.

		// See how many bytes we have.
		nTotBytes := FSEEK( hPriFile, 0, 2 )

		// Reposition to the top of the file.
		FSEEK( hPriFile, 0, 0 )

		// If an initial offset is specified, process it.
		// For RTF's, this is used to skip the header.
		IF VALTYPE( nSkip ) == "N"
			nRead := FSEEK( hPriFile, nSkip )
		ENDIF

		// Dump the delimiters into an array.
		aDelims := ;
			{ ;
			SUBSTR( cClipDelims, 1, 1 ), ;
			SUBSTR( cClipDelims, 2, 1 ), ;
			SUBSTR( cClipDelims, 3, 1 ), ;
			SUBSTR( cClipDelims, 4, 1 )  ;
			}

		// Loop through primary merge file.
		// For RTF's, we want to stop one byte before the end.

		DO WHILE nRead < nTotBytes-1

			++nRead

			cBuffer := FREADSTR( hPriFile, 1 )
			cOrigBuff := cBuffer

			// If we think we're reading a Clipper expression,
			// this determines if we've encountered the next
			// expected part of the sequence.

			lCodeTest := ( cBuffer == aDelims[nCurrCode+1] )

			IF lCodeTest

				// We have met a delimiter for a Clipper expression

				IF nCurrCode < 3
					++nCurrCode
				ELSE

					// We have completed the Clipper expression, so let's
					// evaluate it and dump the result to the output file.

					// First remove extraneous spaces.
					cExpr := ALLTRIM( cExpr )

					// Now macro compile & evaluate it.
					// [NOTE: An error trap for bad expressions would be a nice
					// touch someday -- but this is "proof of concept" right now].

					// eventually, set error handler to catch bad expressions...
					xEval := EVAL( { || &cExpr } )
					// ...reset original error handler

					// If the expression returns a character value, we interpret
					// it as text; otherwise, we just move on.  This allows us to
					// embed directions like "DBSKIP()" in the primary file.

					IF VALTYPE( xEval ) == "C"
						cBuffer := xEval
					ELSE
						cBuffer := ""
					ENDIF

					// Reset the placeholder & flags for the next expression.
					cExpr   := ""
					nCurrCode := 0
					lCodeTest := .F.

				ENDIF

			ELSE

				// It's not a valid delimiter, or is out of sequence.
				// If we were expecting the second in a two-character sequence,
				// we have to treat the first character as simple data now.

				IF nCurrCode == 1 .OR. nCurrcode == 3
					--nCurrCode
					cBuffer := cPrevBuff + cBuffer
				ENDIF

			ENDIF

			// If it wasn't a non-terminating code, we write
			// the data to the output file.
			// We either have "constant" data, or part of a
			// Clipper expression.

			IF !lCodeTest
				IF nCurrCode == 2
					// We're in the process of reading a Clipper expression,
					// so just add to it.
					cExpr += cBuffer
				ELSE
					// We have constant data, so just pass it to the output.
					IF lRTF
						// If we're reading an RTF file, we allow RTF codes
						// in the output, as-is (second parameter is .T.).
						xRTF:Write( cBuffer, .T. )
					ELSE
						// If not an RTF, just write the output directly
						FWRITE( xRTF, cBuffer )
					ENDIF
				ENDIF
			ENDIF

			cPrevBuff := cOrigBuff

		ENDDO

	ENDIF

	FCLOSE( hPriFile )

ENDIF

RETURN NIL
**************************  END  OF RTFMerge()  ***********************

FUNCTION RTFHeader( cFile )
*********************************************************************
* Description:  Return the header portion of an RTF file.  If the
*               return value is empty, it's not an RTF.
*
*               This algorithm is not fool-proof, but it's good
*               enough to handle most real-world RTFs.
*
* Arguments:
* Return:
*
*--------------------------------------------------------------------
* Date       Developer   Comments
* 02/05/97   TRM         Creation
*
*********************************************************************
LOCAL hFile
LOCAL cBuffer := ""
LOCAL cCurrCmd := ""
LOCAL nGrpLevel := 0, nGrpCmd := 0
LOCAL nRealBytes := 0, nRead := 0
LOCAL aHeadCmd := ;
	{ "\rtf", "\colortbl", "\fonttbl", "\info", ;
	  "\filetbl", "\stylesheet", "\revtbl" }

LOCAL nTest := 0

IF IsRTF( cFile )

	hFile := FOPEN( cFile )

	DO WHILE .T.

		cBuffer := FREADSTR(hFile, 1)
		IF cBuffer == ""
			EXIT
		ENDIF
		++nRead

		IF cBuffer $ "{}\; "

			// If currently parsing an RTF command, terminate it.

			IF !EMPTY( cCurrCmd )

				// We're done if this is the first command in a group,
				// but the command is not a standard header command.
				// NOTE: This logic assumes that there is at least one
				// valid header group after the "\rtf..." sequence.
				// It also assumes that the first command in a header group
				// is the group identifier

				IF nGrpLevel < 2 .AND. nGrpCmd == 1 .AND. ;
					ASCAN( aHeadCmd, cCurrCmd ) == 0 .AND. ;
					LEFT( cCurrCmd, 4 ) != aHeadCmd[1]

					// This is not a valid header group, so we assume that
					// the header is complete, and quit.
					EXIT

				ENDIF

				cCurrCmd := ""

			ENDIF

			DO CASE

				CASE cBuffer == "{"
					++nGrpLevel
					nGrpCmd := 0 // count # of commands found in group

				CASE cBuffer == "}"
					--nGrpLevel
					nRealBytes := nRead // Mark spot where last group ended.
					nGrpCmd := 0 // count # of commands found in group

				CASE cBuffer == "\"
					cCurrCmd += cBuffer
					++nGrpCmd

			ENDCASE

		ELSEIF !EMPTY( cCurrCmd )

			// We assume that anything other character is part of the command
			cCurrCmd += cBuffer

		ENDIF

	ENDDO

	// Now read the header bytes from the beginning.
	// Inefficient, but in the vast scheme of things, who really cares?

	cBuffer := ""
	IF nRealBytes > 0
		FSEEK( hFile, 0 )
		cBuffer := FREADSTR( hFile, nRealBytes )
	ENDIF

	FCLOSE( hFile )

ENDIF

RETURN cBuffer
***********************  END OF RTFHeader()  ************************

FUNCTION IsRTF( cFile )
*********************************************************************
* Description:  Determine if a file is an RTF file
* Arguments:
* Return:
*
*--------------------------------------------------------------------
* Date       Developer   Comments
* 02/05/97   TRM         Creation
*
*********************************************************************
LOCAL lRTF := .F., hFile, cBuffer := ""

hFile := FOPEN( cFile )
IF hFile >= 0

	FSEEK( hFile, 0 )
	cBuffer := FREADSTR( hFile, 5 )

	IF cBuffer == "{\rtf"
		lRTF := .T.
	ENDIF
	FCLOSE( hFile )

ENDIF

RETURN lRTF
***********************  END OF IsRTF()  ************************







/*

METHOD RichText:IntlTranslate()
*********************************************************************  COBA 23-11-2019
* Description:  Example of an array that could be used to map
*               high-order characters.
* Arguments:
* Return:
*
*--------------------------------------------------------------------
* Date           Developer       Comments
* 15 april 1998  Paul C. Laney   Creation
*
*********************************************************************
LOCAL aTranslate[128]

AFILL( aTranslate, chr(42) ) // napuni svaki lan sa "*"



aTranslate[01] := "\'c7"
aTranslate[02] := "\'fc"
aTranslate[03] := "\'e9"
aTranslate[04] := "\'e2"
aTranslate[05] := "\'e4"
aTranslate[06] := "\'e0"
aTranslate[07] := "\'e5"
aTranslate[08] := "\'e7"
aTranslate[09] := "\'ea"
aTranslate[10] := "\'eb"
aTranslate[11] := "\'8a"  // 
aTranslate[12] := "\'ef"
aTranslate[13] := "\'ee"
aTranslate[14] := "\'ec"
aTranslate[15] := "\'8e"  // 
aTranslate[16] := "\'c5"
aTranslate[17] := "\'c9"
aTranslate[18] := "\'e6"
aTranslate[19] := "\'c6"
aTranslate[20] := "\'f4"
aTranslate[21] := "\'f6"
aTranslate[22] := "\'f2"
aTranslate[23] := "\'fb"
aTranslate[24] := "\'f9"
aTranslate[25] := "\'ff"
aTranslate[26] := "\'d6"
aTranslate[27] := "\'9a"  // 
aTranslate[28] := "\'a2"
aTranslate[29] := "\'a3"
aTranslate[30] := "\'a5"
aTranslate[31] := "\'9e"  // 
aTranslate[32] := "\'83"
aTranslate[33] := "\'e1"
aTranslate[34] := "\'ed"
aTranslate[35] := "\'f3"
aTranslate[36] := "\'fa"
aTranslate[37] := "\'f1"
aTranslate[38] := "\'d1"
aTranslate[39] := "\'aa"
aTranslate[40] := "\'ba"
aTranslate[41] := "\'bf"

aTranslate[71] := "\'c6"     // 
aTranslate[73] := "\'c8"     // 
aTranslate[81] := "\'d0"     // 

aTranslate[103]:= "\'e6"     // 
aTranslate[105]:= "\'e8"     // 
aTranslate[113]:= "\'f0"     // 


* =8A
* =9A
* =D0
* =F0
* =C8
* =E8
* =C6
* =E6
* =8E
* =9E


::aTranslate := aTranslate

RETURN Self
**********************  END OF IntlTranslate()  *********************
*/





******************************************************** COBA 23-11-2019
METHOD RichText:IntlTranslate()
* FUNCTION RTF_coba_latin_cyrilic(nCHR)
********************************************************
* COBA Systems Software, www.cobasystems.com, 23.11.2019
* TABLA ZA KONVERZIJU LATINICE I IRILICE ZA RTF TEKST
* Konverzija se vri iz CHR koda u HEX kod znaka (slova)
* parametar: nCHR = chr kod znaka (slova)
* return   : cHEX = hex kod znaka (slova)

LOCAL aTranslate[250]  // do 248
AFILL( aTranslate, chr(42) )   // u svaki lan upii *


**************************************** OVDE PROGRAM NE ULAZI

* Eng.Latin cp238/1250 VELIKA SLOVA
/*
Velika latinina slova UTF8
--------------------------------------
UTF8   CHAR     HEX     OCT     DEC
--------------------------------------
U+0041	A	0x41	\101	&#65;  ili CHR(65)
U+0042	B	0x42	\102	&#66;
U+0043	C	0x43	\103	&#67;
U+0044	D	0x44	\104	&#68;
U+0045	E	0x45	\105	&#69;
U+0046	F	0x46	\106	&#70;
U+0047	G	0x47	\107	&#71;
U+0048	H	0x48	\110	&#72;
U+0049	I	0x49	\111	&#73;
U+004A	J	0x4A	\112	&#74;
U+004B	K	0x4B	\113	&#75;
U+004C	L	0x4C	\114	&#76;
U+004D	M	0x4D	\115	&#77;
U+004E	N	0x4E	\116	&#78;
U+004F	O	0x4F	\117	&#79;
U+0050	P	0x50	\120	&#80;
U+0051	Q	0x51	\121	&#81;
U+0052	R	0x52	\122	&#82;
U+0053	S	0x53	\123	&#83;
U+0054	T	0x54	\124	&#84;
U+0055	U	0x55	\125	&#85;
U+0056	V	0x56	\126	&#86;
U+0057	W	0x57	\127	&#87;
U+0058	X	0x58	\130	&#88;
U+0059	Y	0x59	\131	&#89;
U+005A	Z	0x5A	\132	&#90;
*/

aTranslate[65] := "\'41"    // A
aTranslate[66] := "\'42"    // B
aTranslate[67] := "\'43"    // C
aTranslate[68] := "\'44"    // D
aTranslate[69] := "\'45"    // E
aTranslate[70] := "\'46"    // F
aTranslate[71] := "\'47"    // G
aTranslate[72] := "\'48"    // H
aTranslate[73] := "\'49"    // I
aTranslate[74] := "\'4A"    // J
aTranslate[75] := "\'4B"    // K
aTranslate[76] := "\'4C"    // L
aTranslate[77] := "\'4D"    // M
aTranslate[78] := "\'4E"    // N
aTranslate[79] := "\'4F"    // O
aTranslate[80] := "\'50"    // P
aTranslate[81] := "\'51"    // Q
aTranslate[82] := "\'52"    // R
aTranslate[83] := "\'53"    // S
aTranslate[84] := "\'54"    // T
aTranslate[85] := "\'55"    // U
aTranslate[86] := "\'56"    // V
aTranslate[87] := "\'57"    // W
aTranslate[88] := "\'58"    // X
aTranslate[89] := "\'59"    // Y
aTranslate[90] := "\'5A"    // Z

* Eng.Latin cp238/1250 MALA SLOVA

/*
Mala latinina slova UTF8
--------------------------------------
UTF8   CHAR     HEX     OCT     DEC
--------------------------------------
U+0061	a	0x61	\141	&#97;
U+0062	b	0x62	\142	&#98;
U+0063	c	0x63	\143	&#99;
U+0064	d	0x64	\144	&#100;
U+0065	e	0x65	\145	&#101;
U+0066	f	0x66	\146	&#101;
U+0067	g	0x67	\147	&#103;
U+0068	h	0x68	\150	&#104;
U+0069	i	0x69	\151	&#105;
U+006A	j	0x6A	\152	&#106;
U+006B	k	0x6B	\153	&#107;
U+006C	l	0x6C	\154	&#108;
U+006D	m	0x6D	\155	&#109;
U+006E	n	0x6E	\156	&#110;
U+006F	o	0x6F	\157	&#111;
U+0070	p	0x70	\160	&#112;
U+0071	q	0x71	\161	&#113;
U+0072	r	0x72	\162	&#114;
U+0073	s	0x73	\163	&#115;
U+0074	t	0x74	\164	&#116;
U+0075	u	0x75	\165	&#117;
U+0076	v	0x76	\166	&#118;
U+0077	w	0x77	\167	&#119;
U+0078	x	0x78	\170	&#120;
U+0079	y	0x79	\171	&#121;
U+007A	z	0x7A	\172	&#122;
*/


aTranslate[97] := "\'61"    // a
aTranslate[98] := "\'62"    // b
aTranslate[99] := "\'63"    // c
aTranslate[100] := "\'64"    // d
aTranslate[101] := "\'65"    // e
aTranslate[102] := "\'66"    // f
aTranslate[103] := "\'67"    // g
aTranslate[104] := "\'68"    // h
aTranslate[105] := "\'69"    // i
aTranslate[106] := "\'6A"    // j
aTranslate[107] := "\'6B"    // k
aTranslate[108] := "\'6C"    // l
aTranslate[109] := "\'6D"    // m
aTranslate[110] := "\'6E"    // n
aTranslate[111] := "\'6F"    // o
aTranslate[112] := "\'70"    // p
aTranslate[113] := "\'71"    // q
aTranslate[114] := "\'72"    // r
aTranslate[115] := "\'73"    // s
aTranslate[116] := "\'74"    // t
aTranslate[117] := "\'75"    // u
aTranslate[118] := "\'76"    // v
aTranslate[119] := "\'77"    // w
aTranslate[120] := "\'78"    // x
aTranslate[121] := "\'79"    // y
aTranslate[122] := "\'7A"    // z


**************************************** OVDE PROGRAM NE ULAZI



* Serbian Latin cp238/1250
* Extended character set - VELIKA I MALA SLOVA

/*
Naa dodatna latinina slova UTF8
------------------------------------------------------
UTF8   CHAR     HEX              OCT            DEC
------------------------------------------------------
U+0160		0xC5 0xA0	\305\240	&#352;
U+0161		0xC5 0xA1	\305\241	&#353;
U+0110		0xC4 0x90	\304\220	&#272;
U+0111		0xC4 0x91	\304\221	&#273;
U+010C		0xC4 0x8C	\304\214	&#268;
U+010D		0xC4 0x8D	\304\215	&#269;
U+0106		0xC4 0x86	\304\206	&#262;
U+0107		0xC4 0x87	\304\207	&#263;
U+017D		0xC5 0xBD	\305\275	&#381;
U+017E		0xC5 0xBE	\305\276	&#382;
*/


* koristi se iz tabele za translaciju:
* METHOD RichText:IntlTranslate()
* FUNCTION RTF_coba_latin_cyrilic(nCHR)
******************************************************************
* PUBLIC SERBIANCYRILIC := INI_READ("C","SERBIAN","CYRILIC")
* SERBIANCYRILIC := "YES" or "NO" or "" (default)
******************************************************************
// ZADAJ U MAIN() pre: oRTF:=SetupRTF(cOutFile)
// ako postoji ova varijabla koristi se irilica
// tekst koji se upisuje u RTF je tada irilica
IF TYPE("SERBIANCYRILIC")$"C"
   // ako postoji sve je OK , radi se sa SERBIANCYRILIC
ELSE
   // ako ne postoji pravi se i radi se sa latinicom
   SERBIANCYRILIC := "NO"
ENDIF


***************************************
IF UPPER(SERBIANCYRILIC)=="YES"
***************************************

   // ako postoji PUBLIC SERBIANCYRILIC
   // pie se irilicom i koriste se
   // kodovi za Ǝ  iz tabele
   // za irilicu a ne odavde

ELSE

*  STOP("Ǝ",1)

                                // lat        cyr
   aTranslate[138] := "\'8A"    //  ******** LJ
   aTranslate[208] := "\'D0"    //  ******** R
   aTranslate[200] := "\'C8"    //  ******** I
   aTranslate[198] := "\'C6"    //  ******** 
   aTranslate[142] := "\'8E"    //  ******** 
   aTranslate[154] := "\'9A"    //  ******** lj
   aTranslate[240] := "\'F0"    //  ******** r
   aTranslate[232] := "\'E8"    //  ******** i
   aTranslate[230] := "\'E6"    //  ******** 
   aTranslate[158] := "\'9E"    //  ******** 

   ***************************************************
   * Latinina slova                   
   * irilina slova LJ R  I      lj r  i    
   *
   * IMAJU ISTE DEC, HEX KODOVE KAO IRILICA
   * PA SE ZBOG TOGA NE MOGU ZAJEDNO UPISATI - MEATI
   * MORA SE UPISATI ILI LATINICA ILI IRILICA
   * ZAJEDNO UPISANA OVA SLOVA E BITI POGRENA
   * ZAJEDNO SE MOGU MOGU UPISATI SAMO KAO UNICODE
   ***************************************************
   * "\'8a\'d0\'c8\'c6\'8e \'9a\'f0\'e8\'e6\'9e"
   * "\u1064?\u1026?\u1063?\u1035?\u1046? \u1096?\u1106?\u1095?\u1115?\u1078?"

  ::aTranslate := aTranslate
  RETURN Self

ENDIF



* Serbian Cyrilic cp204/1251 VELIKA SLOVA

/*
Velika irilina slova UTF8
------------------------------------------------------
UTF8   CHAR     HEX              OCT            DEC
------------------------------------------------------
U+0410	A	0xD0 0x90	\320\220	&#1040;
U+0411	B	0xD0 0x91	\320\221	&#1041;
U+0412	V	0xD0 0x92	\320\222	&#1042;
U+0413	G	0xD0 0x93	\320\223	&#1043;
U+0414	D	0xD0 0x94	\320\224	&#1044;
U+0415	E	0xD0 0x95	\320\225	&#1045;
U+0416		0xD0 0x96	\320\226	&#1046;
U+0417	Z	0xD0 0x97	\320\227	&#1047;
U+0418	I	0xD0 0x98	\320\230	&#1048;
U+041A	K	0xD0 0x9A	\320\232	&#1050;
U+041B	L	0xD0 0x9B	\320\233	&#1051;
U+041C	M	0xD0 0x9C	\320\234	&#1052;
U+041D	N	0xD0 0x9D	\320\235	&#1053;
U+041E	O	0xD0 0x9E	\320\236	&#1054;
U+041F	P	0xD0 0x9F	\320\237	&#1055;
U+0420	R	0xD0 0xA0	\320\240	&#1056;
U+0421	S	0xD0 0xA1	\320\241	&#1057;
U+0422	T	0xD0 0xA2	\320\242	&#1058;
U+0423	U	0xD0 0xA3	\320\243	&#1059;
U+0424	F	0xD0 0xA4	\320\244	&#1060;
U+0425	H	0xD0 0xA5	\320\245	&#1061;
U+0426	C	0xD0 0xA6	\320\246	&#1062;
U+0427		0xD0 0xA7	\320\247	&#1063;
U+0428		0xD0 0xA8	\320\250	&#1064;
U+0402		0xD0 0x82	\320\202	&#1026;
U+0408	J	0xD0 0x88	\320\210	&#1032;
U+0409	LJ	0xD0 0x89	\320\211	&#1033;
U+040A	NJ	0xD0 0x8A	\320\212	&#1034;
U+040B		0xD0 0x8B	\320\213	&#1035;
U+040F	D	0xD0 0x8F	\320\217	&#1039;
*/

* aTranslate[192] := "\'C0"        //  = A
  aTranslate[192] := "\uc0\u1040 " //  = A   // za rtf dokument
* aTranslate[192] := "&#1040;"     //  = A   // za html dokument

* aTranslate[193] := "\'C1"        //  = B
  aTranslate[193] := "\uc0\u1041 " //  = B  // za RTF obavezan blanko na kraju broja !

* aTranslate[194] := "\'C2"        //  = V
  aTranslate[194] := "\uc0\u1042 " //  = V

* aTranslate[195] := "\'C3"        //  = G
  aTranslate[195] := "\uc0\u1043 " //  = G

* aTranslate[196] := "\'C4"        //  = D
  aTranslate[196] := "\uc0\u1044 " //  = D

* aTranslate[197] := "\'C5"        //  = E
  aTranslate[197] := "\uc0\u1045 " //  = E

* aTranslate[198] := "\'C6"        //  = 
  aTranslate[198] := "\uc0\u1046 " //  = 

* aTranslate[199] := "\'C7"        //  = Z
  aTranslate[199] := "\uc0\u1047 " //  = Z

* aTranslate[200] := "\'C8"        //  = I
  aTranslate[200] := "\uc0\u1048 " //  = I

* aTranslate[202] := "\'CA"        //  = K
  aTranslate[202] := "\uc0\u1050 " //  = K

* aTranslate[203] := "\'CB"        //  = L
  aTranslate[203] := "\uc0\u1051 " //  = L

* aTranslate[204] := "\'CC"        //  = M
  aTranslate[204] := "\uc0\u1052 " //  = M

* aTranslate[205] := "\'CD"        //  = N
  aTranslate[205] := "\uc0\u1053 " //  = N

* aTranslate[206] := "\'CE"        //  = O
  aTranslate[206] := "\uc0\u1054 " //  = O

* aTranslate[207] := "\'CF"        //  = P
  aTranslate[207] := "\uc0\u1055 " //  = P

* aTranslate[208] := "\'D0"        //  = R
  aTranslate[208] := "\uc0\u1056 " //  = R

* aTranslate[209] := "\'D1"        //  = S
  aTranslate[209] := "\uc0\u1057 " //  = S

* aTranslate[210] := "\'D2"        //  = T
  aTranslate[210] := "\uc0\u1058 " //  = T

* aTranslate[211] := "\'D3"        //  = U
  aTranslate[211] := "\uc0\u1059 " //  = U

* aTranslate[212] := "\'D4"        //  = F
  aTranslate[212] := "\uc0\u1060 " //  = F

* aTranslate[213] := "\'D5"        //  = H
  aTranslate[213] := "\uc0\u1061 " //  = H

* aTranslate[214] := "\'D6"        //  = C
  aTranslate[214] := "\uc0\u1062 " //  = C

* aTranslate[215] := "\'D7"        //  = 
  aTranslate[215] := "\uc0\u1063 " //  = 

* aTranslate[216] := "\'D8"        //  = 
  aTranslate[216] := "\uc0\u1064 " //  = 

* aTranslate[128] := "\'80"        //  = 
  aTranslate[128] := "\uc0\u1026 " //  = 

* aTranslate[163] := "\'A3"        //  = J
  aTranslate[163] := "\uc0\u1032 " //  = J

* aTranslate[138] := "\'8A"        //  = LJ
  aTranslate[138] := "\uc0\u1033 " //  = LJ

* aTranslate[140] := "\'8C"        //  = NJ
  aTranslate[140] := "\uc0\u1034 " //  = NJ

* aTranslate[142] := "\'8E"        //  = 
  aTranslate[142] := "\uc0\u1035 " //  = 

* aTranslate[143] := "\'8F"        //  = D
  aTranslate[143] := "\uc0\u1039 " //  = D

* aTranslate[189] := "\'BD"        //  = Y
  aTranslate[189] := "\uc0\u89 "   //  = Y



* Serbian Cyrilic cp204/1251 MALA SLOVA

/*
Mala irilina slova UTF8
------------------------------------------------------
UTF8   CHAR     HEX              OCT            DEC
------------------------------------------------------
U+0430	a	0xD0 0xB0	\320\260	&#1072;
U+0431	b	0xD0 0xB1	\320\261	&#1073;
U+0432	v	0xD0 0xB2	\320\262	&#1074;
U+0433	g	0xD0 0xB3	\320\263	&#1075;
U+0434	d	0xD0 0xB4	\320\264	&#1076;
U+0435	e	0xD0 0xB5	\320\265	&#1077;
U+0436		0xD0 0xB6	\320\266	&#1078;
U+0437	z	0xD0 0xB7	\320\267	&#1079;
U+0438	i	0xD0 0xB8	\320\270	&#1080;
U+043A	k	0xD0 0xBA	\320\272	&#1082;
U+043B	l	0xD0 0xBB	\320\273	&#1083;
U+043C	m	0xD0 0xBC	\320\274	&#1084;
U+043D	n	0xD0 0xBD	\320\275	&#1085;
U+043E	o	0xD0 0xBE	\320\276	&#1086;
U+043F	p	0xD0 0xBF	\320\277	&#1087;
U+0440	r	0xD0 0xC0	\320\280	&#1088;
U+0441	s	0xD0 0xC1	\320\281	&#1089;
U+0442	t	0xD0 0xC2	\320\282	&#1090;
U+0443	u	0xD0 0xC3	\320\283	&#1091;
U+0444	f	0xD0 0xC4	\320\284	&#1092;
U+0445	h	0xD0 0xC5	\320\285	&#1093;
U+0446	c	0xD0 0xC6	\320\286	&#1094;
U+0447		0xD0 0xC7	\320\287	&#1095;
U+0448		0xD0 0xC8	\320\290	&#1096;
U+0458	j	0xD1 0x98	\321\230	&#1112;
U+0459	lj	0xD1 0x99	\321\231	&#1113;
U+045A	nj	0xD1 0x9A	\321\232	&#1114;
U+045B		0xD1 0x9B	\321\233	&#1115;
U+045F	d	0xD1 0x9F	\321\237	&#1119;
U+0452		0xD1 0x92	\321\222	&#1106;
*/


* aTranslate[224] := "\'E0"          //  = a
  aTranslate[224] := "\uc0\u1072 "   //  = a // za rtf dokument
* aTranslate[224] := "&#1072;"       //  = a // za html dokument

* aTranslate[225] := "\'E1"          //  = b
  aTranslate[225] := "\uc0\u1073 "   //  = b  // za RTF obavezan blanko na kraju broja !

* aTranslate[226] := "\'E2"          //  = v
  aTranslate[226] := "\uc0\u1074 "   //  = v

* aTranslate[227] := "\'E3"          //  = g
  aTranslate[227] := "\uc0\u1075 "   //  = g

* aTranslate[228] := "\'E4"          //  = d
  aTranslate[228] := "\uc0\u1076 "   //  = d

* aTranslate[229] := "\'E5"          //  = e
  aTranslate[229] := "\uc0\u1077 "   //  = e

* aTranslate[230] := "\'E6"          //  = 
  aTranslate[230] := "\uc0\u1078 "   //  = 

* aTranslate[231] := "\'E7"          //  = z
  aTranslate[231] := "\uc0\u1079 "   //  = z

* aTranslate[232] := "\'E8"          //  = i
  aTranslate[232] := "\uc0\u1080 "   //  = i

* aTranslate[234] := "\'EA"          //  = k
  aTranslate[234] := "\uc0\u1082 "   //  = k

* aTranslate[235] := "\'EB"          //  = l
  aTranslate[235] := "\uc0\u1083 "   //  = l

* aTranslate[236] := "\'EC"          //  = m
  aTranslate[236] := "\uc0\u1084 "   //  = m

* aTranslate[237] := "\'ED"          //  = n
  aTranslate[237] := "\uc0\u1085 "   //  = n

* aTranslate[238] := "\'EE"          //  = o
  aTranslate[238] := "\uc0\u1086 "   //  = o

* aTranslate[239] := "\'EF"          //  = p
  aTranslate[239] := "\uc0\u1087 "   //  = p

* aTranslate[240] := "\'F0"          //  = r
  aTranslate[240] := "\uc0\u1088 "   //  = r

* aTranslate[241] := "\'F1"          //  = s
  aTranslate[241] := "\uc0\u1089 "   //  = s

* aTranslate[242] := "\'F2"          //  = t
  aTranslate[242] := "\uc0\u1090 "   //  = t

* aTranslate[243] := "\'F3"          //  = u
  aTranslate[243] := "\uc0\u1091 "   //  = u

* aTranslate[244] := "\'F4"          //  = f
  aTranslate[244] := "\uc0\u1092 "   //  = f

* aTranslate[245] := "\'F5"          //  = h
  aTranslate[245] := "\uc0\u1093 "   //  = h

* aTranslate[246] := "\'F6"          //  = c
  aTranslate[246] := "\uc0\u1094 "   //  = c

* aTranslate[247] := "\'F7"          //  = 
  aTranslate[247] := "\uc0\u1095 "   //  = 

* aTranslate[248] := "\'F8"          //  = 
  aTranslate[248] := "\uc0\u1096 "   //  = 

* aTranslate[144] := "\'90"          //  = 
  aTranslate[144] := "\uc0\u1106 "   //  = 

* aTranslate[188] := "\'BC"          //  = j
  aTranslate[188] := "\uc0\u1112 "   //  = j

* aTranslate[154] := "\'9A"          //  = lj
  aTranslate[154] := "\uc0\u1113 "   //  = lj

* aTranslate[156] := "\'9C"          //  = nj
  aTranslate[156] := "\uc0\u1114 "   //  = nj

* aTranslate[158] := "\'9E"          //  = 
  aTranslate[158] := "\uc0\u1115 "   //  = 

* aTranslate[159] := "\'9F"          //  = d
  aTranslate[159] := "\uc0\u1119 "   //  = d

* aTranslate[190] := "\'BE"          //  = y
  aTranslate[190] := "\uc0\u121 "    //  = y


::aTranslate := aTranslate

RETURN Self

// Objanjenje:
//--------------
// Re COBA upisana u RTF fajl iz Worda
// za eng.slova RTF koristi hex.kodove dobijene iz tabele i
// upisuje ih u source txt fajl kao prevedene u karaktere:
* "COBAA"
// za irilicu RTF koristi decimalne kodove kao za HTML: &#1062;
// i upisuje ih u source txt fajl kao kodove ne prevedene u karaktere:
* "\uc0\u1062 \uc0\u1054 \uc0\u1041 \uc0\u1040 \uc0\u1040 "



********* END FUNCTION RTF_coba_latin_cyrilic() *******



// ZAOBILAZAK TopDown BIBLIOTEKE

***********************************************************************
FUNCTION tdYesNo( cTxt )
***********************************************************************
LOCAL vrati := .F.

   IF  ConfirmBox(,cTxt,"PANJA",XBPMB_YESNO  ,XBPMB_QUESTION ,, ) = XBPMB_RET_YES
       vrati := .T.
   ELSE
       vrati := .F.
   ENDIF

RETURN (vrati)

FUNCTION TdMsg( cTxt )
   MsgBox( cTxt,"STOP" )
RETURN (NIL)

* -----------------------------
* AddOn.prg
* -----------------------------

*   #include "Xbp.ch"
*   #include "Appevent.ch"

#pragma library("XppUi2")

***********************************************************************
// iskljuen rad sa dialog boxom
FUNCTION PC_GetSaveFile( cFile )
// za potrebe programa advokat snimi RTF bez filedialog boxa
RETURN ( cFile )

***********************************************************************
// ukljuen rad sa dialog boxom
FUNCTION _PC_GetSaveFile( cFile, lKeepPath )
LOCAL cCurDrive, cCurDir, oFileDlg

   IF valtype( lKeepPath ) <> "L"
      lKeepPath := .T.
   ENDIF

   cCurDrive := CurDrive()
   cCurDir   := CurDir( cCurDrive + ":" )
   oFileDlg := XbpFileDialog():New()
   oFileDlg:Create( AppDeskTop() )
   cFile := oFileDlg:SaveAs( cFile )
   oFileDlg:Destroy()

   IF lKeepPath
      CurDir( cCurDrive + ":\" + cCurDir )
   ENDIF

RETURN (cFile)

***********************************************************************
FUNCTION PC_DotFile( cName, nSize )
LOCAL nLength

   IF valtype( nSize ) <> "N"
      nSize := 25
   ENDIF
   IF nSize < 10
      nSize := 10
   ENDIF

   nLength := len( cName )
   IF nLength > nSize
      cName := "..." + substr( cName, nLength - 17 )
   ENDIF

RETURN ( cName )











